unit WSSE;

interface

uses SysUtils, InvokeRegistry, SOAPHTTPClient, Types, XSBuiltIns;

const
  IS_OPTN = $0001;
  IS_ATTR = $0010;
  IS_TEXT = $0020;
  IS_REF = $0080;

type
  Id = type WideString;

  AttributedString = class(TRemotable)
  private
    FText: WideString;
    FId: Id;
    FId_Specified: boolean;
    procedure SetId(Index: Integer; const AId: Id);
    function Id_Specified(Index: Integer): boolean;
  published
    property Text: WideString Index(IS_TEXT)read FText write FText;
    property Id: Id Index(IS_ATTR or IS_OPTN)
      read FId write SetId stored Id_Specified;
  end;

  Password = class(AttributedString)
  private
    FType_: WideString;
    FType__Specified: boolean;
    procedure SetType_(Index: Integer; const AWideString: WideString);
    function Type__Specified(Index: Integer): boolean;
  published
    property Type_: WideString Index(IS_ATTR or IS_OPTN)
      read FType_ write SetType_ stored Type__Specified;
  end;

  Username = class(AttributedString)
  end;

  UsernameToken = class(TRemotable)
  private
    FUserName: Username;
    FPassword: Password;
    FId: Id;
  public
    destructor Destroy; override;
  published
    property Username: Username read FUserName write FUserName;
    property Password: Password read FPassword write FPassword;
    property Id: Id Index(IS_ATTR)read FId write FId;
  end;

  Security = class(TSOAPHeader)
  private
    FUserNameToken: UsernameToken;
  public
    destructor Destroy; override;
  published
    property UsernameToken: UsernameToken index(IS_REF)
      read FUserNameToken write FUserNameToken;
  end;

implementation

destructor UsernameToken.Destroy;
begin
  FreeAndNIL(FUserName);
  FreeAndNIL(FPassword);
  inherited Destroy;
end;

procedure AttributedString.SetId(Index: Integer; const AId: Id);
begin
  FId := AId;
  FId_Specified := True;
end;

function AttributedString.Id_Specified(Index: Integer): boolean;
begin
  Result := FId_Specified;
end;

procedure Password.SetType_(Index: Integer; const AWideString: WideString);
begin
  FType_ := AWideString;
  FType__Specified := True;
end;

function Password.Type__Specified(Index: Integer): boolean;
begin
  Result := FType__Specified;
end;

destructor Security.Destroy;
begin
  FreeAndNIL(FUserNameToken);
  inherited Destroy;
end;

const
  NS_SECEXT =
    'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd';
  NS_UTILITY =
    'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd';

initialization

RemClassRegistry.RegisterXSClass(Security, NS_SECEXT, 'Security');
RemClassRegistry.RegisterXSClass(UsernameToken, NS_SECEXT, 'UsernameToken');
RemClassRegistry.RegisterXSClass(Username, NS_SECEXT, 'Username');
RemClassRegistry.RegisterXSClass(Password, NS_SECEXT, 'Password');

RemClassRegistry.RegisterXSInfo(TypeInfo(Id), NS_UTILITY, 'Id');
RemClassRegistry.RegisterXSClass(AttributedString, NS_SECEXT,
  'AttributedString');
RemClassRegistry.RegisterExternalPropName(TypeInfo(Password), 'Type_', 'Type');

end.
